Index: iputils-20210202/arping.c
===================================================================
--- iputils-20210202.orig/arping.c
+++ iputils-20210202/arping.c
@@ -90,6 +90,7 @@ struct run_state {
 		unsolicited:1,
 		senduntagged:1,
 		untagged:1,
+		strict:1,
 		sockraw:1;
 	uint16_t tagged;
 };
@@ -138,6 +139,7 @@ static void usage(void)
 		"  -U            unsolicited ARP mode, update your neighbours\n"
 		"  -A            ARP answer mode, update your neighbours\n"
 		"  -V            print version and exit\n"
+		"  -S            Strict interface checking for replies\n"
 		"  -t            accept any 802.1q tagged packets\n"
 		"  -T <vlan-id>  which 802.1q tag to accept and send\n"
 		"  -u            with -T, dont send tagged packets\n"
@@ -505,6 +507,20 @@ static int recv_pack(struct run_state *c
 
 	clock_gettime(CLOCK_MONOTONIC, &ts);
 
+	/* Strict interface reply checking */
+	/* if ifindex is 0 we dont check. Hopefully 0 is never valid*/
+	if (ctl->strict && ctl->device.ifindex && FROM->sll_ifindex &&
+			ctl->device.ifindex != FROM->sll_ifindex)
+	{
+		int inintvlan = 0;
+		if (!ctl->tagged || !ctl->device.vlan_tag)
+			return 0;
+		netlink_devinfo_query(NULL, &FROM->sll_ifindex,
+				DEVINFO_QUERY_INFO_IFINDEX, &inintvlan, DEVINFO_QUERY_TYPE_VLAN);
+		if (inintvlan != ctl->device.vlan_tag)
+			return 0;
+	}
+
 	/* Filter out wild packets */
 	if (FROM->sll_pkttype != PACKET_HOST &&
 	    FROM->sll_pkttype != PACKET_BROADCAST &&
@@ -1109,7 +1125,7 @@ int main(int argc, char **argv)
 	textdomain (PACKAGE_NAME);
 #endif
 #endif
-	while ((ch = getopt(argc, argv, "h?bfDuUAqc:w:i:s:I:VtT:")) != EOF) {
+	while ((ch = getopt(argc, argv, "h?bfDuUAqc:w:i:Ss:I:VtT:")) != EOF) {
 		switch (ch) {
 		case 'b':
 			ctl.broadcast_only = 1;
@@ -1149,6 +1165,9 @@ int main(int argc, char **argv)
 		case 's':
 			ctl.source = optarg;
 			break;
+		case 'S':
+			ctl.strict = 1;
+			break;
 		case 't':
 			ctl.untagged = 1;
 			break;
